<?php

namespace LakeDrops\DorgFlow;

use Exception;
use GitElephant\Command\ResetCommand;
use GitElephant\Repository;
use LakeDrops\Component\Composer\BaseHandler;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Class Handler.
 *
 * @package LakeDrops\DorgFlow
 */
class Handler extends BaseHandler {

  /**
   * {@inheritdoc}
   */
  public function configId(): string {
    return 'dorgflow';
  }

  /**
   * {@inheritdoc}
   */
  protected function configDefault(): array {
    return [
      'projects' => [],
      'drupalspoons' => [],
    ];
  }

  /**
   * Post install/update event to prepare projects for development.
   */
  public function prepareDevProjects(): void {
    // We only do the fancy stuff for developers.
    if (!$this->isDevMode() || $this->isCiContext()) {
      return;
    }

    $this->init();

    $dorgflow = $this->env->receiveGlobal('DORGFLOW', 'Dorgflow', '0');
    if (empty($dorgflow)) {
      return;
    }

    $installationManager = $this->composer->getInstallationManager();
    foreach ([
               'projects' => 'git.drupal.org:project',
               'drupalspoons' => 'gitlab.com:drupalspoons',
               'selfhosted' => FALSE,
             ] as $type => $url) {
      if ($this->config->readValue($type) === NULL) {
        continue;
      }
      if ($url === FALSE) {
        $url = $this->config->readValue([$type, 'url']);
        $projects = $this->config->readValue([$type, 'projects']);
        $simpleMode = FALSE;
      }
      else {
        $projects = $this->config->readValue($type);
        $simpleMode = TRUE;
      }
      $this->io->write('Dorgflow: Preparing ' . $url . ' packages for development', TRUE);
      foreach ($projects as $project => $specification) {
        [, $projectname] = explode('/', $project);
        if (!$simpleMode) {
          $fullUrl = $url . ':' . $specification['path'];
          $version = $specification['version'];
          if (isset($specification['name'])) {
            $projectname = $specification['name'];
          }
        }
        else {
          $fullUrl = $url;
          $version = $specification;
        }
        $package = $this->getPackage($project);
        if ($package === NULL) {
          continue;
        }
        $path = $installationManager->getInstallPath($package);
        $this->io->write("- $project => $path", TRUE);
        $this->prepareWorkingDirectory($path, $projectname, $version, $fullUrl);
      }
    }
  }

  /**
   * Prepare the working directory of a git based package for dorgflow.
   *
   * @param string $path
   *   Name of the working directory.
   * @param string $projectname
   *   Name of the project.
   * @param string $version
   *   Version to checkout.
   * @param string $url
   *   Base URL to checkout from.
   */
  protected function prepareWorkingDirectory(string $path, string $projectname, string $version, string $url): void {
    $uri = 'git@' . $url . '/' . $projectname . '.git';

    // Git Clone the repository into the working directory.
    $repository = Repository::open($path);
    $repository->init();

    try {
      $origin = $repository->getRemote('origin', FALSE);
      if ($origin && $origin->getFetchURL() === $uri) {
        // Already setup correctly.
        return;
      }
    }
    catch (Exception $ex) {
      // Ignore the exception and conitue setup.
    }

    $fs = new Filesystem();
    $fs->remove($path);
    $fs->mkdir($path);
    $repository->init();
    $repository->addRemote('origin', $uri);
    $repository->getCaller()->execute('config --add remote.origin.fetch "+refs/merge-requests/*/head:refs/remotes/origin/merge-requests/*"');

    $repository->fetch();
    $repository->checkout($version);
    $repository->getCaller()->execute('branch --set-upstream-to origin/' . $version . ' ' . $version);
    $this->io->write('  - completed', TRUE);
  }

}
